package uk.co.christophercormack.netkernel.chartnk.accessors;

import java.awt.Rectangle;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringWriter;

import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

public class JFreeChartAccessor extends StandardAccessorImpl
{
    protected static void chartOutput(INKFRequestContext aContext, JFreeChart chart) throws NKFException, IOException
    {
        if (aContext.source("arg:mimetype", String.class).equals("image/svg+xml"))
        {
            String svg = exportChartAsSVG(chart, new Rectangle(aContext.source("arg:width", Integer.class),
                                                               aContext.source("arg:height", Integer.class)));
            INKFResponse svgResponse = aContext.createResponseFrom(svg);
            svgResponse.setMimeType("image/svg+xml");
        }
        else if (aContext.source("arg:mimetype", String.class).equals("image/png"))
        {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            ChartUtilities.writeChartAsPNG(baos,
                                           chart,
                                           aContext.source("arg:width", Integer.class),
                                           aContext.source("arg:height", Integer.class));
            INKFResponse pngResponse = aContext.createResponseFrom(new ByteArrayRepresentation(baos));
            pngResponse.setMimeType("image/png");
        }
    }
    
    /**
     * Exports a JFreeChart to a SVG file.
     * 
     * @param chart JFreeChart to export
     * @param bounds the dimensions of the viewport
     * @param svgFile the output file.
     * @throws IOException if writing the svgFile fails.
     */
    protected static String exportChartAsSVG(JFreeChart chart, Rectangle bounds) throws IOException
    {
        // Get a DOMImplementation and create an XML document
        DOMImplementation domImpl =
            GenericDOMImplementation.getDOMImplementation();
        Document document = domImpl.createDocument(null, "svg", null);

        // Create an instance of the SVG Generator
        SVGGraphics2D svgGenerator = new SVGGraphics2D(document);

        // draw the chart in the SVG generator
        chart.draw(svgGenerator, bounds);
                
        // Write svg file
        StringWriter sw = new StringWriter();
        svgGenerator.stream(sw, true);
        return sw.toString();
    }
}
