package uk.co.christophercormack.netkernel.chartnk.transreptors;

import java.math.BigDecimal;

import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;

public class CategoryDatasetTransreptor extends StandardTransreptorImpl
{
    public CategoryDatasetTransreptor()
    {
        this.declareThreadSafe();
        this.declareToRepresentation(CategoryDataset.class);
        this.declareName("CategoryDatasetTransreptor");
        this.declareDescription("Transrept anything to a JFreeChart CategoryDataset");
    }
    
    @Override
    public void onTransrept(INKFRequestContext context) throws Exception
    {
        IHDSNode data = context.sourcePrimary(IHDSNode.class);
        
        IHDSNode series = data.getFirstNode("/series");
        String type = (String)series.getFirstValue("@type");
        String key = (String)series.getFirstValue("@key");
        
        CategoryDataset datasetRep;
        
        if (type == null) {
            throw new NKFException("J_FREE_CHART_SERIES_NO_TYPE");
        } else if (key == null) {
            throw new NKFException("J_FREE_CHART_SERIES_NO_KEY");
        } else if (type.equals("category")) {
            datasetRep = new DefaultCategoryDataset();
            IHDSNodeList dataItems = series.getNodes("/series/items/item");
            for (IHDSNode dataItem : dataItems)
            {
                Object value = dataItem.getFirstValue("value");
                Object row = dataItem.getFirstValue("row");
                Object column = dataItem.getFirstValue("column");
                
                if (!(value instanceof String) || (value == null)) {
                    throw new NKFException("J_FREE_CHART_SERIES_INVALID_ITEM", value + " not a String at " + dataItem.getXPath());
                }
                if (!(row instanceof String) || (row == null)) {
                    throw new NKFException("J_FREE_CHART_SERIES_INVALID_ITEM", row + " not a String at " + dataItem.getXPath());
                }
                if (!(column instanceof String) || (column == null)) {
                    throw new NKFException("J_FREE_CHART_SERIES_INVALID_ITEM", column + " not a String at " + dataItem.getXPath());
                }
                
                ((DefaultCategoryDataset)datasetRep).addValue(new BigDecimal((String)value),
                                                              (String)row,
                                                              (String)column);
            }
        } else {
            throw new NKFException("J_FREE_CHART_SERIES_UNKNOWN_TYPE");
        }
        
        context.createResponseFrom(datasetRep);
    }

}
