package uk.co.christophercormack.netkernel.chartnk.transreptors;

import java.math.BigDecimal;

import org.jfree.data.time.Day;
import org.jfree.data.time.Hour;
import org.jfree.data.time.Minute;
import org.jfree.data.time.Month;
import org.jfree.data.time.RegularTimePeriod;
import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.time.Year;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;

public class XYDatasetTransreptor extends StandardTransreptorImpl
{
    public XYDatasetTransreptor()
    {
        this.declareThreadSafe();
        this.declareToRepresentation(XYDataset.class);
        this.declareName("XYDatasetTransreptor");
        this.declareDescription("Transrept anything to a JFreeChart XY dataset");
    }
    
    @Override
    public void onTransrept(INKFRequestContext context) throws Exception
    {
        IHDSNode data = context.sourcePrimary(IHDSNode.class);
        
        IHDSNode series = data.getFirstNode("/series");
        String type = (String)series.getFirstValue("@type");
        String key = (String)series.getFirstValue("@key");
        
        XYDataset datasetRep;
        
        if (type == null) {
            throw new NKFException("J_FREE_CHART_SERIES_NO_TYPE");
        } else if (key == null) {
            throw new NKFException("J_FREE_CHART_SERIES_NO_KEY");
        } else if (type.equals("xy")) {
            XYSeries xySeries = new XYSeries(key);
            IHDSNodeList dataItems = series.getNodes("/series/items/item");
            for (IHDSNode dataItem : dataItems) {
                Object x = dataItem.getFirstValue("x");
                Object y = dataItem.getFirstValue("y");
                
                if (!(x instanceof String) || (x == null)) {
                    throw new NKFException("J_FREE_CHART_SERIES_INVALID_ITEM", x + " not a String at " + dataItem.getXPath());
                }
                if (!(y instanceof String) || (y == null)) {
                    throw new NKFException("J_FREE_CHART_SERIES_INVALID_ITEM", x + " not a String at " + dataItem.getXPath());
                }
                
                xySeries.add(new BigDecimal((String)x), new BigDecimal((String)y));
            }
            datasetRep = new XYSeriesCollection(xySeries);
        } else if (type.equals("period")) {
            TimeSeries timeSeries = new TimeSeries(key);
            
            IHDSNodeList dataItems = series.getNodes("/series/items/item");
            for (IHDSNode dataItem : dataItems)
            {
                IHDSNode dateNode = dataItem.getFirstNode("date");
                Object year = dateNode.getFirstValue("year");
                Object month = dateNode.getFirstValue("month");
                Object day = dateNode.getFirstValue("day");
                Object hour = dateNode.getFirstValue("hour");
                Object minute = dateNode.getFirstValue("minute");
                Object second = dateNode.getFirstValue("second");
                
                Object value = dataItem.getFirstValue("value");
                if (!(value instanceof String) || (value == null)) {
                    throw new NKFException("J_FREE_CHART_SERIES_INVALID_ITEM", value + " not a String at " + dataItem.getXPath());
                }
                
                RegularTimePeriod timePeriod = null;
                
                if (second != null)
                {
                    timePeriod = new Second(Integer.parseInt((String)second),
                                           Integer.parseInt((String)minute),
                                           Integer.parseInt((String)hour),
                                           Integer.parseInt((String)day),
                                           Integer.parseInt((String)month),
                                           Integer.parseInt((String)year));
                }
                else if (minute != null)
                {
                    timePeriod = new Minute(Integer.parseInt((String)minute),
                                            Integer.parseInt((String)hour),
                                            Integer.parseInt((String)day),
                                            Integer.parseInt((String)month),
                                            Integer.parseInt((String)year));
                }
                else if (hour != null)
                {
                    timePeriod = new Hour(Integer.parseInt((String)hour),
                                          Integer.parseInt((String)day),
                                          Integer.parseInt((String)month),
                                          Integer.parseInt((String)year));
                }
                else if (day != null)
                {
                    timePeriod = new Day(Integer.parseInt((String)day),
                                         Integer.parseInt((String)month),
                                         Integer.parseInt((String)year));
                    }
                else if (month != null)
                {
                    timePeriod = new Month(Integer.parseInt((String)month),
                                           Integer.parseInt((String)year));
                }
                else if (year != null)
                {
                    timePeriod = new Year(Integer.parseInt((String)year));
                }
                timeSeries.add(timePeriod, new BigDecimal((String)value));
            }
            
            datasetRep = new TimeSeriesCollection(timeSeries);
        } else {
            throw new NKFException("J_FREE_CHART_SERIES_UNKNOWN_TYPE");
        }
        
        context.createResponseFrom(datasetRep);
    }

}
